/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Test per els Dao
 *
 * @author Macià Sintes Andreu
 */
package org.dobile.db.dao;

import androidx.arch.core.executor.testing.InstantTaskExecutorRule;
import androidx.room.Room;
import android.content.Context;
import androidx.test.platform.app.InstrumentationRegistry;
import androidx.test.ext.junit.runners.AndroidJUnit4;


import org.dobile.db.DobileRoomDatabase;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.SwitchNames;
import org.dobile.db.entity.Scene;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestRule;
import org.junit.runner.RunWith;
import org.mockito.MockitoAnnotations;


import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;

/**
 * Test per la classe DeviceSwitchDao
 */
@RunWith(AndroidJUnit4.class)
public class DeviceSwitchDaoTest {
    private DobileRoomDatabase db;
    private DeviceSwitchDao deviceSwitchDao;
    private SceneDao sceneDao;

    @Rule
    public TestRule rule = new InstantTaskExecutorRule();

    /**
     * Crea una base de dades en memòria per fer els test i  no afectar a les dades guardades a la base
     * de dades persistent. Les dades creades en aquesta base de dades desapareixen quan finalitza el
     * proces.
     * Room comprova que no s'accedeix a la base de dades des del thread principal, no hi deixa accedir
     * des d'aquest thread. Per fer les proves es deshabilita aquesta comprovació amb alloMainThreadQueries()
     */
    @Before
    public  void createDb(){

        MockitoAnnotations.initMocks(this);

        Context context = InstrumentationRegistry.getInstrumentation().getTargetContext();
        db = Room.inMemoryDatabaseBuilder(context, DobileRoomDatabase.class)
                .allowMainThreadQueries().build();
        deviceSwitchDao = db.deviceSwitchDao();
        sceneDao = db.sceneDao();
    }

    /**
     * Tanca la base de dades
     *
     * @throws IOException
     */
    @After
    public void closeDb() throws IOException {
        db.close();
    }

    @Test
    public void insert() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch dev = new DeviceSwitch("Llum cuina", "192.168.1.156", 3, scene.getId(),deviceNames);

        deviceSwitchDao.insertDevice(dev);


        List<DeviceSwitch> listDevices = deviceSwitchDao.getListDevicesForScene(scene.getId());

        assertEquals(1, listDevices.size());
    }

    @Test
    public void deleteDevice() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch dev = new DeviceSwitch("Llum cuina", "192.168.1.156", 3, scene.getId(), deviceNames);
        deviceSwitchDao.insertDevice(dev);

        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();

        dev = listDevices.get(0);

        assertEquals(1, listDevices.size());

        deviceSwitchDao.deleteDevice(dev);

        listDevices = deviceSwitchDao.getListAllDevices();

        assertEquals(0, listDevices.size());
    }

    @Test
    public void deleteAll() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch dev = new DeviceSwitch("Llum cuina", "192.168.1.156", 3, scene.getId(), deviceNames);

        deviceSwitchDao.insertDevice(dev);

        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();

        assertEquals(1, listDevices.size());

        deviceSwitchDao.deleteAll();

        listDevices = deviceSwitchDao.getListAllDevices();

        assertEquals(0, listDevices.size());
    }

    @Test
    public void update() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch dev = new DeviceSwitch("Llum cuina", "192.168.1.156", 3, scene.getId(), deviceNames);

        deviceSwitchDao.insertDevice(dev);

        List<DeviceSwitch> listDevices = deviceSwitchDao.getListDevicesForScene(scene.getId());

        dev = listDevices.get(0);
        dev.setName("Focus cuina");
        deviceSwitchDao.updateDevice(dev);

        listDevices = deviceSwitchDao.getListDevicesForScene(scene.getId());
        DeviceSwitch deviceUpdated = listDevices.get(0);

        assertEquals("Focus cuina", deviceUpdated.getName());
    }

    @Test
    public void getDevice() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch dev = new DeviceSwitch("Llum cuina", "192.168.1.156", 3, scene.getId(), deviceNames);

        deviceSwitchDao.insertDevice(dev);
        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();
        dev = listDevices.get(0);

        DeviceSwitch deviceSelected = deviceSwitchDao.getDevice(dev.getId());

        assertEquals(dev.getId(), deviceSelected.getId());
    }

    @Test
    public void getListDevicesForScene() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames1 = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames1.add(devName1);
        deviceNames1.add(devName2);
        deviceNames1.add(devName3);

        List<SwitchNames> deviceNames2 = new ArrayList<>();

        SwitchNames dev2Name1 = new SwitchNames("Lampara 1");
        SwitchNames dev2Name2 = new SwitchNames("Lampara 2");
        SwitchNames dev2Name3 = new SwitchNames("Lampara 3");

        deviceNames2.add(dev2Name1);
        deviceNames2.add(dev2Name2);
        deviceNames2.add(dev2Name3);

        List<SwitchNames> deviceNames3 = new ArrayList<>();

        SwitchNames dev3Name1 = new SwitchNames("Llum 1");
        SwitchNames dev3Name2 = new SwitchNames("Focus 2");
        SwitchNames dev3Name3 = new SwitchNames("Lampara 3");

        deviceNames3.add(dev3Name1);
        deviceNames3.add(dev3Name2);
        deviceNames3.add(dev3Name3);

        List<SwitchNames> deviceNames4 = new ArrayList<>();

        SwitchNames dev4Name1 = new SwitchNames("Ventilador 1");
        SwitchNames dev4Name2 = new SwitchNames("Ventilador 2");
        SwitchNames dev4Name3 = new SwitchNames("Ventilador 3");

        deviceNames3.add(dev4Name1);
        deviceNames3.add(dev4Name2);
        deviceNames3.add(dev4Name3);



        DeviceSwitch dev1 = new DeviceSwitch("Llum cuina", "192.168.1.156", 3, scene.getId(), deviceNames1);
        DeviceSwitch dev2 = new DeviceSwitch("Focus cuina", "192.168.1.157", 3, scene.getId(), deviceNames2);
        DeviceSwitch dev3 = new DeviceSwitch("Llums altres cuina", "192.168.1.158", 3, scene.getId(), deviceNames3);
        DeviceSwitch dev4 = new DeviceSwitch("Ventiladors cuina", "192.168.1.159", 3, scene.getId(), deviceNames4);

        deviceSwitchDao.insertDevice(dev1);
        deviceSwitchDao.insertDevice(dev2);
        deviceSwitchDao.insertDevice(dev3);
        deviceSwitchDao.insertDevice(dev4);

        List<DeviceSwitch> listDevices = deviceSwitchDao.getListDevicesForScene(scene.getId());

        assertEquals(4, listDevices.size());
    }

    @Test public void insertDeviceWithDeviceNamesTest() throws Exception{
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);



        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch dev = new DeviceSwitch("Llum cuina", "192.168.1.156", 3, scene.getId(), deviceNames);

        deviceSwitchDao.insertDeviceWithSwitchNames(dev);

        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();
        assertEquals("Llista dispositius creats es igual a 1", 1, listDevices.size());

        DeviceSwitch devSelected = listDevices.get(0);
        List<SwitchNames> devNamesSelected = deviceSwitchDao.getSwitchNames(devSelected.getId());

        assertEquals("Llista de noms de dispositius igual a 3", 3, devNamesSelected.size());
    }

    @Test public void updateDeviceNamesTest() throws Exception{
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch dev = new DeviceSwitch("Llum cuina", "192.168.1.156", 3, scene.getId(), deviceNames);




        deviceSwitchDao.insertDeviceWithSwitchNames(dev);

        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();
        assertEquals("Llista dispositius creats es igual a 1", 1, listDevices.size());

        DeviceSwitch devSelected = listDevices.get(0);
        List<SwitchNames> devNamesSelected = deviceSwitchDao.getSwitchNames(devSelected.getId());

        devNamesSelected.get(1).setName("Llum central");
        deviceSwitchDao.updateSwitchNames(devNamesSelected);
        List<SwitchNames> devNamesUpdated = deviceSwitchDao.getSwitchNames(devSelected.getId());

        assertEquals("Llista de noms actualitzada", "Llum central", devNamesUpdated.get(1).getName());
    }

    @Test public void deleteDeviceWithDeviceNamesTest() throws Exception{
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);



        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch dev = new DeviceSwitch("Llum cuina", "192.168.1.156", 3, scene.getId(), deviceNames);

        deviceSwitchDao.insertDeviceWithSwitchNames(dev);

        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();
        assertEquals("Llista dispositius creats es igual a 1", 1, listDevices.size());

        DeviceSwitch devSelected = listDevices.get(0);
        int idDevice = devSelected.getId();

        List<SwitchNames> devNamesToDelete = deviceSwitchDao.getSwitchNames(idDevice);
        assertEquals("Llista noms abans de esborrar dispositiu és igual a 3", 3, devNamesToDelete.size());


        deviceSwitchDao.deleteDevice(devSelected);

        List<SwitchNames> devNamesDeleted = deviceSwitchDao.getSwitchNames(idDevice);
        assertEquals("Llista noms despres de esborrar dispositiu és igual a 0", 0, devNamesDeleted.size());
    }
}
