/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * L'app utilitza el patrò Model-View-ViewModel recomanat per Android. Aquest patrò està compost de:
 *
 *  - View: La interfície gràfica que interactua amb l'usuari
 *  - ViewModel: Connecta les dades amb la interfície gràfica
 *  - DataModel: La capa de dades, en aquest cas la base de dades
 *
 *  El repositori és la connexió del ViewModel amb el DataModel, és a dir amb la base de dades. Les operacions amb la
 *  base de dades es fan en un thread separat del principal, mitjançant AsycTask, per no interferir en la interacció de
 *  l'usuari amb la interfície gràfica
 *
 * @author Macià Sintes Andreu
 **/

package org.dobile.db;


import android.app.Application;
import android.os.AsyncTask;

import java.util.List;

import androidx.lifecycle.LiveData;
import android.util.Log;

import org.dobile.db.dao.DeviceACDao;
import org.dobile.db.dao.DeviceSwitchDao;
import org.dobile.db.dao.SwitchNamesDao;
import org.dobile.db.dao.SceneDao;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.DeviceAC;
import org.dobile.db.entity.SwitchNames;
import org.dobile.db.entity.Scene;

/**
 * Repositori de l'app que connecta la base de dades amb la capa ViewModel
 */
public class DobileRepository {

    private static final String LOG_TAG = DobileRepository.class.getSimpleName();

    //Daos de la base de dades
    private SceneDao mSceneDao;
    private DeviceSwitchDao mDeviceSwitchDao;
    private DeviceACDao mDeviceACDao;
    private SwitchNamesDao mSwitchNamesDao;

    /*Llista amb tots els dispositius de tipus interruptor*/
    private LiveData<List<DeviceSwitch>> mAllSwitchDevices;

    /*Llista amb tots els dispositius de tipus aire acondicionat*/
    private LiveData<List<DeviceAC>> mAllACDevices;

    /*Llista amb totes les escenes*/
    private LiveData<List<Scene>> mAllScenes;

    /*Llista amb tots els noms d'interruptors*/
    LiveData<List<SwitchNames>> mAllSwitchNames;

    /**
     * Constructor del repositori. Obté els Daos de la base de dades i las llistes
     *
     * @param application
     */
    public DobileRepository(Application application) {
        DobileRoomDatabase db = DobileRoomDatabase.getDatabase(application);
        mSceneDao = db.sceneDao();
        mDeviceSwitchDao = db.deviceSwitchDao();
        mSwitchNamesDao = db.switchNamesDao();
        mDeviceACDao = db.deviceACDao();
        mAllScenes = mSceneDao.getAllScenes();
        mAllSwitchDevices = mDeviceSwitchDao.getAllDevices();
        mAllACDevices = mDeviceACDao.getAllDevices();
        mAllSwitchNames = mSwitchNamesDao.getAllSwitchNames();
    }

    /**
     * Constructor alternatiu que accepta una database. S'utilitza en els tests per a poder
     * passar una base de dades que es pugui executar en el thread principal. Room per defecte
     * no permet que les base de dades s'executin en el thread principal per no afectar a la
     * interfície gràfica
     *
     * @param database - Base de dades creada per realitzar els tests. Es poden fer operacions en el
     *           thread principal
     */
    public DobileRepository(DobileRoomDatabase database){
        DobileRoomDatabase db = database;
        mSceneDao = db.sceneDao();
        mDeviceSwitchDao = db.deviceSwitchDao();
        mSwitchNamesDao = db.switchNamesDao();
        mAllScenes = mSceneDao.getAllScenes();
        mAllSwitchDevices = mDeviceSwitchDao.getAllDevices();
        mAllACDevices = mDeviceACDao.getAllDevices();
        mAllSwitchNames = mSwitchNamesDao.getAllSwitchNames();
    }


    /**
     * Retorna totes les escenes
     *
     * @return LiveData amb totes les escenes
     */
    public LiveData<List<Scene>> getAllScenes() {
        return mAllScenes;
    }

    /**
     * Retorna tots els dispositius de tipus aire acondicionat
     *
     * @return LiveData amb tots els dispositius de tipus aire acondicionat
     */
    public LiveData<List<DeviceAC>> getAllACDevices() {
        return mAllACDevices;
    }

    /**
     * Retorna tots els dispositius de tipus interruptor
     *
     * @return LiveData amb tots els dispositius de tipus interruptor
     */
    public LiveData<List<DeviceSwitch>> getAllSwitchDevices() {
        return mAllSwitchDevices;
    }


    /**
     * Retorna tots els noms d'interruptors
     *
     * @return LiveData amb tots els noms d'interruptors
     */
    public LiveData<List<SwitchNames>> getmAllSwitchNames() {return mAllSwitchNames;};

    /**
     * Inserta una escena
     *
     * @param scene
     */
    public void insertScene (Scene scene) {

        new insertSceneAsyncTask(mSceneDao).execute(scene);
    }

    /**
     * Esborra una escena
     *
     * @param scene
     */
    public void deleteScene(Scene scene)  {
        new deleteSceneAsyncTask(mSceneDao).execute(scene);
    }

    /**
     * Actualitza una escena
     *
     * @param scene
     */
    public void updateScene(Scene scene){
        new updateSceneAsyncTask(mSceneDao).execute(scene);
    }


    /**
     * Inserta un dispositiu de tipus interruptor
     *
     * @param device
     */
    public void insertDeviceSwitch(DeviceSwitch device) {
        Log.d(LOG_TAG, "Insertar dispositiu a Repositori");
        new insertDevSwitAsyncTask(mDeviceSwitchDao).execute(device);
    }

    /**
     * Esborra un dispositiu de tipus interruptor
     *
     * @param device
     */
    public void deleteDeviceSwitch(DeviceSwitch device)  {
        new deleteDevSwitAsyncTask(mDeviceSwitchDao).execute(device);
    }

    /**
     * Actualitza un dispositiu de tipus interruptor amb la seva llista de noms d'interruptors
     *
     * @param device
     * @param namesSwitches
     */
    public void updateDeviceSwitch(DeviceSwitch device, List<String> namesSwitches){
        new updateDevSwitAsyncTask(mDeviceSwitchDao).execute(device, namesSwitches);
    }

    /**
     * Retorna la llista de dispositius de tipus interruptor de una escena
     *
     * @param sceneId
     * @return LiveData amb la llista de dispositius de tipus interruptor de la escena
     */
    public LiveData<List<DeviceSwitch>> getDevicesSwitchForScene(int sceneId){
        LiveData<List<DeviceSwitch>> lDevices;

        lDevices = mDeviceSwitchDao.getDevicesForScene(sceneId);

        return lDevices;
    }

    /**
     * Inserta un dispositiu de tipus aire acondicionat
     *
     * @param device
     */
    public void insertDeviceAC(DeviceAC device) {
        Log.d(LOG_TAG, "Insertar dispositiu a Repositori");
        new insertDevACAsyncTask(mDeviceACDao).execute(device);
    }

    /**
     * Esborra un dispositiu de tipus aire acondicionat
     *
     * @param device
     */
    public void deleteDeviceAC(DeviceAC device)  {
        new deleteDevACAsyncTask(mDeviceACDao).execute(device);
    }

    /**
     * Actualitza un dispositiu de tipus aire acondicionat
     *
     * @param device
     */
    public void updateDeviceAC(DeviceAC device){
        new updateDevACAsyncTask(mDeviceACDao).execute(device);
    }

    /**
     * Retorna un dispositiu de tipus aire acondicionat encapsulat en un LiveData
     * @param idDevice
     * @return dispositiu aire acondicionat encapsulat en un LiveData
     */
    public LiveData<DeviceAC> getDeviceAC(int idDevice) {
        LiveData<DeviceAC> device = mDeviceACDao.getDeviceLiveData(idDevice);

        return device;
    }

    /**
     * Retorna la llista de dispositius de tipus aire acondicionat de una escena
     *
     * @param sceneId
     * @return LiveData amb la llista de dispositius de tipus aire acondicionat de la escena
     */
    public LiveData<List<DeviceAC>>  getDevicesACForScene(int sceneId){
        LiveData<List<DeviceAC>> lDevices;

        lDevices = mDeviceACDao.getDevicesForScene(sceneId);

        return lDevices;
    }

    public LiveData<List<SwitchNames>> getSwNamesForDevice(int deviceId) {
        LiveData<List<SwitchNames>> lNames;

        lNames = mSwitchNamesDao.getNamesForDevice(deviceId);

        return lNames;
    }



    //Les operacions per esborrar, insertar i actualitzar a la base de dades s'han de fer en un
    //thread separat del principal per no interferir amb les interaccions amb l'usuari de la
    //interficie gràfica. Els mètodes sequents implementen aquestes operacions amb AsyncTask

    /**
     * AsyncTask per deleteScene
     */
    private static class deleteSceneAsyncTask extends AsyncTask<Scene, Void, Void> {
        private SceneDao scDao;

        deleteSceneAsyncTask(SceneDao dao) {
            scDao = dao;
        }

        @Override
        protected Void doInBackground(final Scene... params) {
            scDao.deleteScene(params[0]);
            return null;
        }
    }

    /**
     * AsyncTask per insertScene
     */
    private static class insertSceneAsyncTask extends AsyncTask<Scene, Void, Void> {

        private SceneDao scDao;

        insertSceneAsyncTask(SceneDao dao) {
            scDao = dao;
        }

        @Override
        protected Void doInBackground(final Scene... params) {
            scDao.insert(params[0]);
            return null;
        }
    }

    /**
     * AsyncTask per updateScene
     */
    private static class updateSceneAsyncTask extends AsyncTask<Scene, Void, Void> {
        private SceneDao scDao;

        updateSceneAsyncTask(SceneDao dao) {
            scDao = dao;
        }

        @Override
        protected Void doInBackground(final Scene... params) {
            scDao.update(params[0]);
            return null;
        }
    }

    /**
     * AsyncTask per insertDeviceSwitch
     */
    private static class insertDevSwitAsyncTask extends AsyncTask<DeviceSwitch, Void, Void> {

        private DeviceSwitchDao devDao;


        insertDevSwitAsyncTask(DeviceSwitchDao dao) {
            devDao = dao;
        }

        @Override
        protected Void doInBackground(final DeviceSwitch... params) {
            Log.d(LOG_TAG, "Insertar dispositiu a doInBackground");
            DeviceSwitch dev = params[0];

            devDao.insertDeviceWithSwitchNames(params[0]);

            return null;

        }

    }


    /**
     * AsyncTask per deleteDeviceSwitch
     */
    private static class deleteDevSwitAsyncTask extends AsyncTask<DeviceSwitch, Void, Void> {
        private DeviceSwitchDao devDao;

        deleteDevSwitAsyncTask(DeviceSwitchDao dao) {
            devDao = dao;
        }

        @Override
        protected Void doInBackground(final DeviceSwitch... params) {
            devDao.deleteDevice(params[0]);
            return null;
        }
    }

    /**
     * AsyncTask per updateDeviceSwitch
     */
    private static class updateDevSwitAsyncTask extends AsyncTask<Object, Void, Void> {
        private DeviceSwitchDao devDao;

        updateDevSwitAsyncTask(DeviceSwitchDao dao) {
            devDao = dao;
        }

        @Override
        protected Void doInBackground(final Object... params) {
            DeviceSwitch device = (DeviceSwitch)params[0];
            List<String> devNames = (List<String>) params[1];
            devDao.updateDeviceWithSwitchNames(device, devNames);
            return null;
        }
    }

    /**
     * AsyncTask per insertDeviceAC
     */
    private static class insertDevACAsyncTask extends AsyncTask<DeviceAC, Void, Void> {

        private DeviceACDao devDao;


        insertDevACAsyncTask(DeviceACDao dao) {
            devDao = dao;
        }

        @Override
        protected Void doInBackground(final DeviceAC... params) {
            Log.d(LOG_TAG, "Insertar dispositiu a doInBackground");
            DeviceAC dev = params[0];

            devDao.insertDevice(dev);

            return null;

        }

    }

    /**
     * AsyncTask per deleteDeviceAC
     */
    private static class deleteDevACAsyncTask extends AsyncTask<DeviceAC, Void, Void> {
        private DeviceACDao devDao;

        deleteDevACAsyncTask(DeviceACDao dao) {
            devDao = dao;
        }

        @Override
        protected Void doInBackground(final DeviceAC... params) {
            devDao.deleteDevice(params[0]);
            return null;
        }
    }

    /**
     * AsyncTask per updateDeviceSwitch
     */
    private static class updateDevACAsyncTask extends AsyncTask<Object, Void, Void> {
        private DeviceACDao devDao;

        updateDevACAsyncTask(DeviceACDao dao) {
            devDao = dao;
        }

        @Override
        protected Void doInBackground(final Object... params) {
            DeviceAC device = (DeviceAC)params[0];
            devDao.updateDevice(device);
            return null;
        }
    }

}
