/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Per la persistència s'utilitza la llibreria Room. Aquesta llibreria aporta una capa d'abstracció sobre SQLite. Conté tres
 * componentes principals:
 *
 *  - Database: És el punt d'acces principal a la base de dades
 *  - Entity: Representa una taula
 *  - DAO: Conté els mètodes per accedir a la base de dades
 *
 * Database de Room que proporciona la connexió amb la base de dates
 *
 * @author Macià Sintes Andreu
 **/

package org.dobile.db;

import androidx.sqlite.db.SupportSQLiteDatabase;
import androidx.room.Database;
import androidx.room.Room;
import androidx.room.RoomDatabase;
import androidx.room.migration.Migration;
import android.content.Context;
import android.os.AsyncTask;
import androidx.annotation.NonNull;

import org.dobile.db.dao.DeviceACDao;
import org.dobile.db.dao.DeviceSwitchDao;
import org.dobile.db.dao.SwitchNamesDao;
import org.dobile.db.dao.SceneDao;
import org.dobile.db.entity.DeviceAC;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.SwitchNames;
import org.dobile.db.entity.Scene;


/**
 * Database de la llibreria Room que proporciona la connexió amb la base de dades
 */
@Database(entities = {Scene.class, DeviceSwitch.class, DeviceAC.class, SwitchNames.class}, version = 13, exportSchema = false)
public abstract class DobileRoomDatabase extends RoomDatabase {


    //Daos per interactuar amb les taules de la base de dades
    public abstract SceneDao sceneDao();
    public abstract DeviceSwitchDao deviceSwitchDao();
    public abstract DeviceACDao deviceACDao();
    public abstract SwitchNamesDao switchNamesDao();

    //Instància del Database
    private static volatile DobileRoomDatabase INSTANCE;

    private static RoomDatabase.Callback sRoomDatabaseCallback =
            new RoomDatabase.Callback(){

                //Es crida quan la base de dades s'obre
                @Override
                public void onOpen (@NonNull SupportSQLiteDatabase db){
                    super.onOpen(db);
                    new PopulateDbAsync(INSTANCE).execute();
                }
            };

    /**
     * Retorna la instància del database. En el cas de que no existeixi la crea
     *
     * @param context
     * @return Instància del Database
     */
    static DobileRoomDatabase getDatabase(final Context context) {
        if (INSTANCE == null) {
            synchronized (DobileRoomDatabase.class) {
                if (INSTANCE == null) {
                    //Crea la base de dades
                    INSTANCE = Room.databaseBuilder(context.getApplicationContext(),
                            DobileRoomDatabase.class, "dobile_database")
                            //Esborra i reconstrueix la base de dades
                            //si no hi ha cap objecte Migration
                            .fallbackToDestructiveMigration()
                            .addCallback(sRoomDatabaseCallback)
                            .build();
                }
            }
        }
        return INSTANCE;
    }

    /**
     * Objectes Migration que s'encarreguen de gestionar el canvi de versió de la base
     * de dades, quan es modifica el seu esquema
     */

    static final Migration MIGRATION_1_2 = new Migration(1, 2) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            // Since we didn't alter the table, there's nothing else to do here.
        }
    };

    /**
     * Objectes Migration que s'encarreguen de gestionar el canvi de versió de la base
     * de dades, quan es modifica el seu esquema
     */
    static final Migration MIGRATION_2_3 = new Migration(2, 3) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE users "
                    + " ADD COLUMN last_update INTEGER");
        }
    };


    /**
     * Classe privada per insertar nous objectes a la base de dades inicialment, per
     * fer proves
     */
    private static class PopulateDbAsync extends AsyncTask<Void, Void, Void> {

        private final SceneDao mDao;
        private final DeviceSwitchDao dDao;

        PopulateDbAsync(DobileRoomDatabase db) {
            mDao = db.sceneDao();
            dDao = db.deviceSwitchDao();
        }

        @Override
        protected Void doInBackground(final Void... params) {
            mDao.deleteAll();
            //Scene sceneCuina = new Scene("Cuina");
            //mDao.insertSwitch(sceneCuina);
            //Scene sceneSalon = new Scene("Salon");
            //mDao.insertSwitch(sceneSalon);

            dDao.deleteAll();
            //DeviceSwitch dev = new DeviceSwitch("Aire 12", "234", "AA", sceneCuina.getId());
            //dDao.insertDeviceSwitch(dev);
            //dev = new DeviceSwitch("Aire 15", "678", "AA",  sceneCuina.getId());
            //dDao.insertDeviceSwitch(dev);

            return null;
        }
    }
}
