/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Per la persistència s'utilitza la llibreria Room. Aquesta llibreria aporta una capa d'abstracció sobre SQLite. Conté tres
 * componentes principals:
 *
 *  - Database: És el punt d'acces principal a la base de dades
 *  - Entity: Representa una taula
 *  - DAO: Conté els mètodes per accedir a la base de dades
 *
 * DAO amb els mètodes per accedir a la taula de dispositius de tipus aire acondicionat
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.db.dao;

import androidx.lifecycle.LiveData;
import androidx.room.Dao;
import androidx.room.Delete;
import androidx.room.Insert;
import androidx.room.OnConflictStrategy;
import androidx.room.Query;
import androidx.room.Update;
import org.dobile.db.entity.DeviceAC;
import java.util.List;

/**
 * DAO amb els mètodes per accedir a la taula de dispositius de tipus aire acondicionat
 */
@Dao
public  abstract class DeviceACDao {

    /**
     * Inserta un dispositiu de tipus aire acondicionat a la base de dades
     * @param device - Dispositiu de tipus aire acondicionat
     * @return Id del dispositiu
     */
    @Insert(onConflict = OnConflictStrategy.IGNORE)
    public abstract long insertDevice(DeviceAC device);


    /**
     * Esborra un dispositiu de tipus aire acondicionat
     * @param device- Dispositiu a esborrar
     */
    @Delete
    public abstract void deleteDevice(DeviceAC device);

    /**
     * Retorna un dispositiu de tipus aire acondicionat amb id demanat
     * @param id - Id del dispositiu
     * @return Dispositiu amb id demanat
     */
    @Query("SELECT * FROM devices_ac WHERE id = :id")
    public abstract DeviceAC getDevice(int id);

    /**
     * Retorna un dispositiu de tipus aire acondicionat amb id demanat encapsulat dins un LiveData
     * @param id - Id del dispositiu
     * @return Dispositiu amb id demanat, encapsulat dins un LiveData
     */
    @Query("SELECT * FROM devices_ac WHERE id = :id")
    public abstract LiveData<DeviceAC>  getDeviceLiveData(int id);

    /**
     * Esborra tots els dispositius
     */
    @Query("DELETE FROM devices_ac")
    public abstract void deleteAll();

    /**
     * Retorna tots els dispositius en un LiveData que conté una List
     * @return LiveData amb tots els dispositiius
     */
    @Query("SELECT * from devices_ac ORDER BY name ASC")
    public abstract LiveData<List<DeviceAC>> getAllDevices();

    /**
     * Retorna tots els dispositius en un List. Mètode creat per poder fer tests i comprovar
     * el funcionament
     * @return List amb tots els dispositius
     */
    @Query("SELECT * from devices_ac ORDER BY name ASC")
    public abstract List<DeviceAC> getListAllDevices();

    /**
     * Actualitza un dispositiu de tipus aire acondicionat
     * @param device
     */
    @Update(onConflict = OnConflictStrategy.REPLACE)
    public abstract void updateDevice(DeviceAC device);


    /**
     * Retorna tots els dispositius de tipus aire acondicionat d'una escena en un LiveData
     * @param id de l'escena
     * @return LiveData que conté una List amb els dispositius de l'escena amb id demanat
     */
    @Query("SELECT * FROM devices_ac WHERE scene_id IS :id")
    public abstract LiveData<List<DeviceAC>> getDevicesForScene(int id);

    /**
     * Retorna tots els dispositius de tipus aire acondicionat d'una escena en un List. Mètode creat per fer tests per comprovar el
     * funcionament
     * @param id de l'escena
     * @return List amb els dispositius de l'escena amb id demanat
     */
    @Query("SELECT * FROM devices_ac WHERE scene_id IS :id")
    public abstract List<DeviceAC> getListDevicesForScene(int id);

}

