/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Per la persistència s'utilitza la llibreria Room. Aquesta llibreria aporta una capa d'abstracció sobre SQLite. Conté tres
 * componentes principals:
 *
 *  - Database: És el punt d'acces principal a la base de dades
 *  - Entity: Representa una taula
 *  - DAO: Conté els mètodes per accedir a la base de dades
 *
 * DAO amb els mètodes per accedir a la taula de dispositius de tipus interruptor
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.db.dao;

import androidx.lifecycle.LiveData;
import androidx.room.Dao;
import androidx.room.Delete;
import androidx.room.Insert;
import androidx.room.OnConflictStrategy;
import androidx.room.Query;
import androidx.room.Update;

import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.SwitchNames;

import java.util.ArrayList;
import java.util.List;

/**
 * DAO amb els mètodes per accedir a la taula de dispositius de tipus interruptor
 */
@Dao
public  abstract class DeviceSwitchDao {

    /**
     * Inserta un dispositiu de tipus interruptor a la base de dades
     * @param device - Dispositiu a insertar
     * @return Id del dispositiu
     */
    @Insert(onConflict = OnConflictStrategy.IGNORE)
    public abstract long insertDevice(DeviceSwitch device);

    /**
     * Inserta una llista de noms de interruptors
     * @param switchNames - Llista amb els noms d'interruptors d'un dispositiu
     */
    @Insert(onConflict = OnConflictStrategy.IGNORE)
    public abstract void insertSwitchNames(List<SwitchNames> switchNames);

    /**
     * Esborra una llista de noms d'interruptors
     * @param listToDelete - Llista de noms d'interruptors d'un dispositiu a esborrar
     */
    @Delete
    public abstract void deleteSwitchNames(List<SwitchNames> listToDelete);


    /**
     * Esborra un dispositiu
     * @param device - Dispositiu a esborrar
     */
    @Delete
    public abstract void deleteDevice(DeviceSwitch device);

    /**
     * Retorna un dispositiu de tipus interruptor amb id demanat
     * @param id - Id del dispositiu a retornar
     * @return
     */
    @Query("SELECT * FROM devices_switch WHERE id = :id")
    public abstract DeviceSwitch getDevice(int id);

    /**
     * Retorna un dispositiu de tipus interruptor en una llista, amb id demanat
     * @param id - Id del dispositiu
     * @return List amb un dispositiu amb id demanat
     */
    @Query("SELECT * FROM switch_names WHERE device_id = :id")
    public abstract List<SwitchNames> getSwitchNames(int id);

    /**
     * Esborra tots els dispositius tipus interruptor
     */
    @Query("DELETE FROM devices_switch")
    public abstract void deleteAll();

    /**
     * Retorna tots els dispositius de tipus interruptor en un LiveData que conté una List
     * @return LiveData amb tots els dispositiius de tipus interruptor
     */
    @Query("SELECT * from devices_switch ORDER BY name ASC")
    public abstract LiveData<List<DeviceSwitch>> getAllDevices();

    /**
     * Retorna tots els dispositius de tipus interruptor en un List. Mètode creat per poder fer tests i comprovar
     * el funcionament
     * @return List amb tots els dispositius de tipus interruptor
     */
    @Query("SELECT * from devices_switch ORDER BY name ASC")
    public abstract List<DeviceSwitch> getListAllDevices();

    /**
     * Actualitza un dispositiu de tipus interruptor
     * @param device - Dispositiu a actualitzar
     */
    @Update(onConflict = OnConflictStrategy.REPLACE)
    public abstract void updateDevice(DeviceSwitch device);

    /**
     * Actualizta una llista de noms d'interruptors d'un dispositiu
     * @param switchNames
     */
    @Update(onConflict = OnConflictStrategy.REPLACE)
    public abstract void updateSwitchNames(List<SwitchNames> switchNames);

    /**
     * Retorna tots els dispositius de tipus interruptor de una escena en un LiveData
     * @param id de l'escena
     * @return LiveData que conté una List amb els dispositius de tipus interruptor de l'escena amb id demanat
     */
    @Query("SELECT * FROM devices_switch WHERE scene_id IS :id")
    public abstract LiveData<List<DeviceSwitch>> getDevicesForScene(int id);

    /**
     * Retorna tots els dispositius de tipus interruptor de una escena en un List. Mètode creat per fer tests per comprovar el
     * funcionament
     * @param id de l'escena
     * @return List amb els dispositius de l'escena amb id demanat
     */
    @Query("SELECT * FROM devices_switch WHERE scene_id IS :id")
    public abstract List<DeviceSwitch> getListDevicesForScene(int id);

    /**
     * Inserta un dispositiu de tipus interruptor amb la llista de noms de interruptors inclosa. Primer inserta el dispositiu
     * i obté el seu id. Després inserta l'id del dispositiu a tots els noms d'interruptors de la llista.
     * Finalment inserta la llista de noms d'interruptors
     * @param device
     */
    public void insertDeviceWithSwitchNames(DeviceSwitch device){
        int idDevice = (int)insertDevice(device);
        List<SwitchNames> switchNames = device.getSwitchNames();
        for (int i = 0; i < switchNames.size(); i++){
            switchNames.get(i).setDeviceId(idDevice);
        }
        insertSwitchNames(switchNames);
    }


    /**
     * Retorna un dispositiu amb la seva llista de noms de interruptors. Primer obté el dispositiu
     * demanat, despres la llista de noms d'interruptors. Finalment inserta la llista de noms d'interruptors
     * al dispositiu
     * @param id - Id del dispositiu
     * @return Dispositiu amb la seva llista de noms de interruptors
     */
    public DeviceSwitch getDeviceWithSwitchNames(int id){
        DeviceSwitch device = getDevice(id);
        List<SwitchNames> switchNames = getSwitchNames(id);
        device.setSwitchNames(switchNames);
        return device;
    }

    /**
     * Actualitza un dispositiu amb la seva llista de noms d'interruptors.
     * En el cas que la llista de noms sigui més petita que l'anterior, s'esborren els
     * noms que sobren. En el cas que sigui major es creen els nous objectes de nom d'interruptor
     * i s'actualitzen la resta. En el cas que sigui igual només s'actualitzen els noms de la llista.
     * @param device
     * @param strSwitchNames List de Strings amb els noms d'interruptors a actualitzar
     */
    public void updateDeviceWithSwitchNames(DeviceSwitch device, List<String> strSwitchNames){
        List<SwitchNames> switchNames = device.getSwitchNames();

        int switchNamSize = switchNames.size();
        int strSwitchNamSize = strSwitchNames.size();

        //La nova llista de noms de dispositius és menor que la original, per tant s'esborren els que sobren
        if (switchNamSize > strSwitchNamSize){
            int to = switchNamSize - 1;
            List <SwitchNames> listToDelete = getSubList(switchNames, strSwitchNamSize, to );
            deleteSwitchNames(listToDelete);


            to = strSwitchNamSize - 1;
            List <SwitchNames> listToUpdate = getSubList(switchNames, 0 , to);
            for (int i = 0; i < strSwitchNamSize;i++){
                listToUpdate.get(i).setName(strSwitchNames.get(i));
            }
            updateSwitchNames(listToUpdate);

        }else if ( switchNamSize < strSwitchNamSize){
            //La nova llista de noms de dispositius és major que la original, per tant es creen nous noms de dispositius
            int size = strSwitchNamSize - switchNamSize;
            List<SwitchNames> listToAdd = new ArrayList<>(size);

            for(int i = switchNamSize; i < strSwitchNamSize; i++){
                SwitchNames devName = new SwitchNames(strSwitchNames.get(i), device.getId());
                listToAdd.add(devName);
            }

            insertSwitchNames(listToAdd);

            for (int i = 0; i < switchNamSize;i++){
                switchNames.get(i).setName(strSwitchNames.get(i));
            }
            updateSwitchNames(switchNames);

        } else if (switchNamSize == strSwitchNamSize){
            //No es modifica el nombre de dispositius. No s'esborren ni es creen
            for (int i = 0; i < switchNamSize;i++){
                switchNames.get(i).setName(strSwitchNames.get(i));
            }
            updateSwitchNames(switchNames);
        }

        updateDevice(device);
    }

    /**
     * Retorna una part de una llista
     * @param list - Llista original de la qual s'extreu una part
     * @param from - Posició de la llista a partir de la qual s'extreu la subllista, Inclosa
     * @param to - Posició de la llista fins la qual s'extreu la subllista, Inclosa
     * @return - Llista parcial que s'extreu
     */
    private List getSubList(List<SwitchNames> list, int from, int to){
        int sizeList = to - from + 1;
        List<SwitchNames> subList = new ArrayList<>(sizeList);

        for (int i = from; i <= to; i++){
            subList.add(list.get(i));
        }

        return subList;

    }


}

