/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Per la persistència s'utilitza la llibreria Room. Aquesta llibreria aporta una capa d'abstracció sobre SQLite. Conté tres
 * componentes principals:
 *
 *  - Database: És el punt d'acces principal a la base de dades
 *  - Entity: Representa una taula
 *  - DAO: Conté els mètodes per accedir a la base de dades
 *
 * DAO amb els mètodes per accedir a la taula dels noms de dispositius
 *
 * @author Macià Sintes Andreu
 **/
package org.dobile.db.dao;

import androidx.lifecycle.LiveData;
import androidx.room.Dao;
import androidx.room.Delete;
import androidx.room.Insert;
import androidx.room.OnConflictStrategy;
import androidx.room.Query;
import androidx.room.Update;

import org.dobile.db.entity.SwitchNames;

import java.util.List;

/**
 * DAO amb els mètodes per accedir a la taula dels noms de dispositius
 */
@Dao
public interface SwitchNamesDao {

    /**
     * Inserta un nom d'interruptor
     *
     * @param switchNames
     */
    @Insert(onConflict = OnConflictStrategy.IGNORE)
    void insert(SwitchNames switchNames);

    /**
     * Inserta una llista de noms d'interuptors
     *
     * @param switchNames
     */
    @Insert(onConflict = OnConflictStrategy.IGNORE)
    void insert(List<SwitchNames> switchNames);

    /**
     * Esborra una llisa de noms d'interruptors
     *
     * @param switchNames
     */
    @Delete
    void deleteSwitchNames(SwitchNames switchNames);

    /**
     * Actualitza una llista de noms d'interruptors
     *
     * @param switchNames
     */
    @Update(onConflict = OnConflictStrategy.REPLACE)
    void updateSwitchNames(List<SwitchNames> switchNames);

    /**
     * Retorna tots els noms d'interruptors en un LiveData
     *
     * @return
     */
    @Query("SELECT * from switch_names ORDER BY name ASC")
    LiveData<List<SwitchNames>> getAllSwitchNames();

    /**
     * Retorna tots els noms d'interruptors en una List, per facilitar els tests
     * @return
     */
    @Query("SELECT * from switch_names ORDER BY name ASC")
    List<SwitchNames> getListAllSwitchNames();

    /**
     * Retorna tots els noms d'interruptos d'un dispositiu en un LiveData
     *
     * @param deviceId - Id del dispositiu
     * @return LiveData amb els noms d'interruptors del dispositiu amb id deviceId
     */
    @Query("SELECT * FROM switch_names WHERE device_id IS :deviceId")
    LiveData<List<SwitchNames>> getNamesForDevice(int deviceId);


    /**
     * Retorna tots els noms d'interruptos d'un dispositiu en un List, per facilitar els tests
     *
     * @param deviceId - Id del dispositiu
     * @return List amb els noms d'interruptors del dispositiu amb id deviceId
     */
    @Query("SELECT * FROM switch_names WHERE device_id IS :deviceId")
    List<SwitchNames> getListNamesForDevice(int deviceId);
}
