/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Per la persistència s'utilitza la llibreria Room. Aquesta llibreria aporta una capa d'abstracció sobre SQLite. Conté tres
 * componentes principals:
 *
 *  - Database: És el punt d'acces principal a la base de dades
 *  - Entity: Representa una taula
 *  - DAO: Conté els mètodes per accedir a la base de dades
 *
 *
 * Entitat amb la taula dels dispositius
 *
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.db.entity;

import androidx.room.ColumnInfo;
import androidx.room.Entity;
import androidx.room.ForeignKey;
import androidx.room.Ignore;
import androidx.room.PrimaryKey;
import androidx.annotation.NonNull;

import org.dobile.model.Device;

import java.io.Serializable;
import java.util.List;

/**
 * Entity que representa la taula devices_switch que conté els dispositius de tipus aire acondicionat
 */
@Entity(tableName = "devices_switch",
        foreignKeys =
        @ForeignKey(entity = Scene.class,
                parentColumns = "id",
                childColumns = "scene_id",
                onDelete = ForeignKey.CASCADE))
public class DeviceSwitch implements Device, Serializable {

    /**
     * Clau primària id, generada automàticament
     */
    @PrimaryKey(autoGenerate = true)
    private int id;

    /**
     * Nom del dispositiu
     */
    @NonNull
    @ColumnInfo(name = "name")
    private String name;

    /**
     * Ip del dispositiu
     */
    @NonNull
    @ColumnInfo(name = "ip")
    private String ip;


    /**
     * Nombre de interruptors
     */
    @ColumnInfo(name = "num_switches")
    private int numSwitches;

    /**
     * Id de l'escena a la qual pertany el dispositiu
     */
    @ColumnInfo(name = "scene_id")
    private int sceneId;


    @Ignore
    private List<SwitchNames> switchNames;

    /**
     * Constructor del dispositiu, sense la llista de noms d'interruptors
     *
     * @param name
     * @param ip
     * @param numSwitches
     * @param sceneId
     */
    public DeviceSwitch(String name, String ip, int numSwitches, int sceneId) {
        this.name = name;
        this.ip = ip;
        this.numSwitches = numSwitches;
        this.sceneId = sceneId;
    }

    /**
     * Constructor del dispositiu amb la llista de noms d'interruptors
     *
     * @param name
     * @param ip
     * @param numSwitches
     * @param sceneId
     * @param switchNames
     */
    public DeviceSwitch(String name, String ip, int numSwitches, int sceneId, List<SwitchNames> switchNames) {
        this.name = name;
        this.ip = ip;
        this.numSwitches = numSwitches;
        this.sceneId = sceneId;
        this.switchNames = switchNames;
    }

    /**
     * Retorna l'id del dispositiu
     *
     * @return Id del dispositiu
     */
    public int getId() {
        return id;
    }

    /**
     * Canvia l'id del dispositiu
     *
     * @param id
     */
    public void setId(int id) {
        this.id = id;
    }

    /**
     * Retorna el nom del dispositiu
     *
     * @return Nom del dispositiu
     */
    public String getName(){return this.name;}

    /**
     * Canvia el nom del dispositiu
     *
     * @param name
     */
    public void setName(@NonNull String name) {
        this.name = name;
    }

    /**
     * Retorna la ip del dispositiu
     *
     * @return Ip del dispositiu
     */
    @NonNull
    public String getIp() {
        return ip;
    }

    /**
     * Canvia la ip del dispositiu
     *
     * @param ip
     */
    public void setIp(@NonNull String ip) {
        this.ip = ip;
    }


    /**
     * Retorna el número d'interruptors del dispositiu. Entre 1 i 8
     *
     * @return Número d'interruptors del dispositiu
     */
    public int getNumSwitches() {
        return numSwitches;
    }

    /**
     * Canvia el nombre de interruptors. Entre 1 i 8
     * @param numSwitches
     */
    public void setNumSwitches(int numSwitches) {
        this.numSwitches = numSwitches;
    }


    /**
     * Retorna l'id de l'escena a la qual pertany el dispositiu
     *
     * @return Id de l'escena a la qual pertany el dispositiu
     */
    public int getSceneId() {
        return sceneId;
    }

    /**
     * Canvia l'id de l'escena a la qual pertany el dispositiu
     *
     * @param sceneId
     */
    public void setSceneId(int sceneId) {
        this.sceneId = sceneId;
    }

    /**
     * Retorna la llista de noms dels interruptors
     *
     * @return Llista amb els noms dels interruptors
     */
    public List<SwitchNames> getSwitchNames() {
        return switchNames;
    }

    /**
     * Canvia la llista de noms d'interruptors
     *
     * @param deviceNames
     */
    public void setSwitchNames(List<SwitchNames> deviceNames){
        this.switchNames = deviceNames;
    }
}