/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Activitat que mostra l'escena seleccionada
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.ui;

import androidx.appcompat.view.menu.MenuBuilder;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProviders;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import androidx.annotation.Nullable;;
import com.google.android.material.floatingactionbutton.ExtendedFloatingActionButton;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.appcompat.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.EditText;
import android.widget.RadioButton;

import org.dobile.R;
import org.dobile.db.entity.DeviceAC;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.Scene;
import org.dobile.viewmodel.DeviceACViewModel;
import org.dobile.viewmodel.DeviceSwitchViewModel;
import org.dobile.viewmodel.SceneViewModel;

import java.lang.reflect.Method;
import java.util.List;

/**
 * Mostra l'escena seleccionada amb la llista de dispositius que té. Més les opcions
 * per crear, esborrar i editar dispositius
 */
public class SceneActivity extends AppCompatActivity {

    /*Id de l'escena*/
    private int sceneId;

    /*Entitat que representa l'escena*/
    private Scene scene;

    /*ViewModel de les escenes*/
    private SceneViewModel mSceneViewModel;

    /*ViewModel dels dispositius*/
    private DeviceSwitchViewModel deviceSwitchViewModel;
    private DeviceACViewModel deviceACViewModel;

    private static final String LOG_TAG = SceneActivity.class.getSimpleName();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_scene);
        Toolbar toolbar = findViewById(R.id.toolbarEscene);
        setSupportActionBar(toolbar);

        //Intent que ha creat l'activity. S'extreu l'escena seleccionada i enviada per l'intent
        Intent intent = getIntent();
        if (scene == null){
            scene = (Scene)intent.getSerializableExtra("sceneSelected");
            getSupportActionBar().setTitle(scene.getName());
        }


        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        //RecyclerView per mostrar la llista de dispositius de tipus interruptor
        RecyclerView recyclerViewSwitches = findViewById(R.id.devswitchesrecview);
        final DevSwitchesListAdapter adapterSwitches = new DevSwitchesListAdapter(this);
        recyclerViewSwitches.setAdapter(adapterSwitches);
        recyclerViewSwitches.setLayoutManager(new LinearLayoutManager(this));
        recyclerViewSwitches.setNestedScrollingEnabled(false);

        //RecyclerView per mostrar la llista de dispositius de tipus aire acondicionat
        RecyclerView recyclerViewAC = findViewById(R.id.devacrecview);
        final DevACListAdapter adapterAC = new DevACListAdapter(this);
        recyclerViewAC.setAdapter(adapterAC);
        recyclerViewAC.setLayoutManager(new LinearLayoutManager(this));
        recyclerViewAC.setNestedScrollingEnabled(false);

        //ViewModels de les escenes i dispositius
        mSceneViewModel = ViewModelProviders.of(this).get(SceneViewModel.class);
        deviceSwitchViewModel = ViewModelProviders.of(this).get(DeviceSwitchViewModel.class);
        deviceACViewModel = ViewModelProviders.of(this).get(DeviceACViewModel.class);

        Log.d(LOG_TAG, "Id escena: " + scene.getId());
        Log.d(LOG_TAG, "Nom escena: " + scene.getName());



         //Crea un nou observer sobre el LiveData que conté la llista de dispositius de tipus interruptor de l'escena.
         setDevSwitchesObserver(adapterSwitches);


         //Crea un nou observer sobre el LiveData que conté la llista de dispositius de tipus aire acondicionat de l'escena.
         setDevACObserver(adapterAC);


        /*Butó per crear nous dispositius*/
        final ExtendedFloatingActionButton fab = findViewById(R.id.fabAddDevice);

        /**
         * OnClickListener per crear nous dispositius. Arranca una nova activity
         * per entrar els paràmetres del nou dispositiu
         */
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Log.d(LOG_TAG, "Nou dispositiu: " + fab.toString());
                showNewDeviceDialog();

                /*Intent intentNewDevice = new Intent(SceneActivity.this, NewDeviceActivity.class);
                intentNewDevice.putExtra(MainActivity.SCENE_SELECTED, scene.getId());
                startActivity(intentNewDevice);*/
            }
        });

    }

    /**
     * Crea el menu de l'activity
     *
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_scene, menu);

        // To display icon on overflow menu
        if(menu instanceof MenuBuilder){
            MenuBuilder m = (MenuBuilder) menu;
            m.setOptionalIconsVisible(true);
        }
        return true;
    }


    /**
     * Accions que es duen a terme quan es selecciona una opció del menú
     *
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();

        switch (id){
            case R.id.menu_add_device:  //Afegir nou dispositiu a l'escena
                showNewDeviceDialog();
                return true;

            case R.id.menu_delete_scene:    //Esborrar escena
                Log.d("", "Escena seleccionada: " + scene.getName());
                mSceneViewModel.deleteScene(scene);
                finish();
                return true;

            case R.id.menu_edit_scene:    //Editar escena. Mostra el diàleg per canviar de nom
                showEditSceneDialog();

                break;

            case R.id.menu_scene_help:       //Mostra l'ajuda
                DialogFragment helpFragment = new HelpFragment();
                helpFragment.show(getSupportFragmentManager(), "help");
                break;

            default:
                return super.onOptionsItemSelected(item);
        }

        return super.onOptionsItemSelected(item);
    }

    /**
     * Mostra el diàlge per editar l'escena. Per canviar el nom de l'escena
     */
    public void showEditSceneDialog() {
        // get prompts.xml view
        LayoutInflater layoutInflater = LayoutInflater.from(SceneActivity.this);
        View viewDialog = layoutInflater.inflate(R.layout.dialog_new_scene, null);
        AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(SceneActivity.this);
        alertDialogBuilder.setView(viewDialog);

        final EditText editText = (EditText) viewDialog.findViewById(R.id.dialogEdit);
        // setup a dialog window
        alertDialogBuilder.setCancelable(false)
                .setPositiveButton("Guardar", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        String sceneName = editText.getText().toString();
                        scene.setName(sceneName);
                        Log.d(LOG_TAG, sceneName);
                        mSceneViewModel.updateScene(scene);
                        getSupportActionBar().setTitle(sceneName);
                    }
                })
                .setNegativeButton("Cancel·lar",
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                dialog.cancel();
                            }
                        });

        AlertDialog alert = alertDialogBuilder.create();
        alert.setTitle("Editar escena");
        alert.show();

    }

    /**
     * Mostra el diàleg per crear un nou dispositiu. Primer es demana el tipus de dispositiu
     */
    public void showNewDeviceDialog() {
        // get prompts.xml view
        LayoutInflater layoutInflater = LayoutInflater.from(SceneActivity.this);
        View viewDialog = layoutInflater.inflate(R.layout.dialog_new_device, null);
        AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(SceneActivity.this);
        alertDialogBuilder.setView(viewDialog);

        final RadioButton buttonSwitch = (RadioButton) viewDialog.findViewById(R.id.radio_switch_device);
        buttonSwitch.setChecked(true);
        // setup a dialog window
        alertDialogBuilder.setCancelable(false)
                .setPositiveButton("Crear dispositiu", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        Intent intentNewDevice;
                        if (buttonSwitch.isChecked()) {
                            intentNewDevice = new Intent(SceneActivity.this, NewDeviceSwitchActivity.class);
                        }else {
                            intentNewDevice = new Intent(SceneActivity.this, NewDeviceACActivity.class);
                        }

                        intentNewDevice.putExtra(MainActivity.SCENE_SELECTED, scene.getId());
                        startActivity(intentNewDevice);

                    }
                })
                .setNegativeButton("Cancel·lar", //butó cancelar
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                dialog.cancel();
                            }
                        });

        // create an alert dialog
        AlertDialog alert = alertDialogBuilder.create();
        alert.setTitle("Nou dispositiu");
        alert.show();

    }

    /**
     * Crea un nou observer sobre el LiveData que conté la llista de dispositius de tipus interruptor de l'escena.
     * Qualssevol canvi a la base de dades actualitza automàticament la View
     */
    private void setDevSwitchesObserver(final DevSwitchesListAdapter adapterSwitches){

        deviceSwitchViewModel.getDevicesForScene(scene.getId()).observe(this, new Observer<List<DeviceSwitch>>() {
            @Override
            public void onChanged(@Nullable final List<DeviceSwitch> devices) {

                adapterSwitches.setDevices(devices);
            }
        });
    }

    /**
     * Crea un nou observer sobre el LiveData que conté la llista de dispositius de tipus aire acondicionat de l'escena.
     * Qualssevol canvi a la base de dades actualitza automàticament la View
     */
    private void setDevACObserver(final DevACListAdapter adapterAC){
       deviceACViewModel.getDevicesForScene(scene.getId()).observe(this, new Observer<List<DeviceAC>>() {
           @Override
           public void onChanged(@Nullable final List<DeviceAC> devices) {

               adapterAC.setDevices(devices);
           }
       });
   }


}
