/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Adapter del RecyclerView per mostra la llista d'escenes
 *
 * @author Macià Sintes Andreu
 */


package org.dobile.ui;


import android.content.Context;
import android.content.Intent;
import androidx.recyclerview.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.TextView;

import org.dobile.R;
import org.dobile.db.entity.Scene;

import java.util.List;

/**
 * ListAdapter del RecyclerView per mostrar la llista d'escenes
 */
public class SceneListAdapter extends RecyclerView.Adapter<SceneListAdapter.SceneViewHolder> {

    private static final String LOG_TAG = SceneListAdapter.class.getSimpleName();
    private final LayoutInflater mInflater;
    private List<Scene> mScenes;

    /**
     * Classe interna ViewHolder que representa un sol ítem de la llista amb les dades a mostrar, en aquest cas el nom de l'escena
     */
    class SceneViewHolder extends RecyclerView.ViewHolder implements OnClickListener {
        private final TextView sceneItemView;
        final SceneListAdapter mAdapter;
        public static final String SCENE_ID = "org.dobile.ui.scenelistadapter.extra.MESSAGE";

        /**
         * Constructor del ViewHolder
         *
         * @param itemView
         * @param adapter
         */
        private SceneViewHolder(View itemView, SceneListAdapter adapter) {
            super(itemView);
            sceneItemView = itemView.findViewById(R.id.textSceneItem);
            mAdapter = adapter;
            sceneItemView.setOnClickListener(this);
        }

        /**
         * Acció que s'executa quan es clicka un ítem de la llista
         *
         * @param v View del item clickat
         */
        @Override
        public void onClick(View v) {

            //Es recupera la posició de l'ítem clickat
            int mPosition = getLayoutPosition();

            //Amb la posició s'extreu l'escena de la llista d'escenes
            Scene scene = mScenes.get(mPosition);

            Log.d(LOG_TAG, "Nom escena: " + scene.getName());
            Log.d(LOG_TAG, "Id escena: " + scene.getId());

            //Es crea un nou intent per mostrar l'escena seleccionada i els seus dispositius
            Context context = v.getContext();
            Intent intent = new Intent(context, SceneActivity.class);
            intent.putExtra("sceneSelected", scene);
            context.startActivity(intent);

        }
    }


    /**
     * Constructor de SceneListAdapter
     *
     * @param context
     */
    SceneListAdapter(Context context) {
        mInflater = LayoutInflater.from(context);
    }

    /**
     * Crea un nou ViewHolder inflant el layout scene_item que contindrà un sol ítem
     *
     * @param parent
     * @param viewType
     * @return
     */
    @Override
    public SceneViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View itemView = mInflater.inflate(R.layout.scene_item, parent, false);
        return new SceneViewHolder(itemView, this);
    }

    /**
     * Assigna les dades, en aquest cas el nom de l'escena, a l'ítem que està en una determinada posició
     *
     * @param holder ViewHolder de l'ítem
     * @param position Posició de l'ítem a la llista
     */
    @Override
    public void onBindViewHolder(SceneViewHolder holder, int position) {
        if (mScenes != null) {
            Scene current = mScenes.get(position);
            holder.sceneItemView.setText(current.getName());

        }
    }

    /**
     * Assigna la llista d'escenes i notifica a l'Observer que les dades han canviat
     * @param scenes
     */
    void setScenes(List<Scene> scenes){
        mScenes = scenes;
        notifyDataSetChanged();
    }

    /**
     * Retorna el número d'escenes que conté la llista
     *
     * @return El número d'escenes de la llista o 0 si la llista és null
     */
    @Override
    public int getItemCount() {
        if (mScenes != null)
            return mScenes.size();
        else return 0;
    }

    /**
     * Retorna l'escena de una posició determinada de la llista
     *
     * @param position Posició de l'escena a la llista
     * @return Escena de la posició demanada
     */
    public Scene getSceneAtPosition (int position) {
        return mScenes.get(position);
    }
}